package com.web.api.controller;

import com.web.api.constant.CommonConstant;
import com.web.api.exception.BusinessException;
import com.web.api.model.DataResult;
import com.web.api.model.po.Content;
import com.web.api.utils.OssClientUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.util.List;

/**
 * @author 王玉鹏
 * @version 1.0
 * @className UploadController
 * @description TODO
 * @date 2019/12/23 15:16
 */
@RestController
@RequestMapping(CommonConstant.PLATFORM_URL_PREFIX+"/upload")
@Slf4j
@Api(value = "文件上传",tags = "文件上传")
public class UploadController {

    @Resource
    private OssClientUtils ossClientUtils;

    @Value("${oss.upload.image}")
    private String imageDir;

//    @Resource
//    private ContentDao contentDao;


    @PostMapping(value = "/uploadImage")
    @ApiOperation(value = "图片上传", httpMethod = "POST",notes = "图片上传")
    public DataResult uploadImage(final MultipartFile file) {
        try {
            if (file == null || file.getSize() <= 0) {
                log.info("发生错误{}","文件为空");
                throw new BusinessException("0","文件为空");
            }
            String keyName = ossClientUtils.uploadImg2Oss(file,imageDir);
            log.info("上传文件：{}至阿里oss成功",keyName);
            String imgUrl = ossClientUtils.getImgUrl(keyName);
            return DataResult.ok(imgUrl);
        } catch (Exception e) {
            log.info("发生错误{}",e);
            return DataResult.fail(-1,e.getMessage());
        }
    }
    @PostMapping(value = "/deleteImage")
    @ApiOperation(value = "图片上传", httpMethod = "POST",notes = "图片上传")
    public DataResult deleteImage(final String keyName) {
        try {
            if (StringUtils.isEmpty(keyName)) {
                log.info("文件名不能为空");
                throw new BusinessException("-1","文件名不能为空");
            }
            ossClientUtils.delFile(imageDir+keyName);
            log.info("从阿里oss删除文件：{}成功",keyName);
            return DataResult.ok("删除成功");
        } catch (Exception e) {
            log.info("删除文件发生错误{}",e);
            return DataResult.fail(-1,e.getMessage());
        }
    }

/*    @PostMapping(value = "/updatemage")
    @ApiOperation(value = "图片更新", httpMethod = "POST",notes = "图片上传")
    public DataResult updatemage() {
        try {
           List<Content> list = contentDao.selectAll();
            list.forEach(e->{
                if (!StringUtils.isEmpty(e.getImagePath())){
                    String keyName = e.getImagePath().substring(e.getImagePath().indexOf("/image/"),e.getImagePath().indexOf("?")).replace("/image/","").trim();
                    System.out.println(keyName);
                    String imgUrl = ossClientUtils.getImgUrl(keyName);
                    e.setImagePath(imgUrl);
                    System.out.println(imgUrl);
                    System.out.println("更新结果："+ contentDao.updateByPrimaryKey(e));
                }

            });


            return DataResult.ok();
        } catch (BusinessException e) {
            log.info("发生错误{}",e);
            return DataResult.fail(0,e.getMessage());
        }
    }*/
}
