package com.web.api.controller.answer;

import com.web.api.constant.CommonConstant;
import com.web.api.model.DataResult;
import com.web.api.model.po.AnswerJob;
import com.web.api.service.AnswerJobService;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;

/**
 * @author 王玉鹏
 * @version 1.0
 * @className AnswerJobController
 * @description TODO
 * @date 2022/8/15 14:58
 */
@RestController
@RequestMapping(CommonConstant.ANSWER_URL_PREFIX + "/api/job")
@Api(value = "定时答题", tags = "定时答题")
@Slf4j
public class AnswerJobController {

    @Autowired
    private AnswerJobService answerJobService;

    /**
     * 查询所有AnswerJob数据的方法
     *
     * @param value
     * @return
     */
    @PostMapping(value = "/find")
    public DataResult find(@RequestBody AnswerJob value) {
        try {
            return DataResult.ok(answerJobService.find(value));
        } catch (Exception e) {
            log.error("发生错误：{}", e);
            return DataResult.fail(e.getMessage());
        }
    }

    /**
     * 通过id查询AnswerJob数据的方法
     *
     * @param id
     * @return
     */
    @GetMapping(value = "/get/{id}")
    public DataResult findOne(@PathVariable(name = "id") Integer id) {

        try {
            return DataResult.ok(answerJobService.findOne(id));
        } catch (Exception e) {
            log.error("发生错误：{}", e);
            return DataResult.fail(e.getMessage());
        }
    }

    /**
     * 插入AnswerJob属性不为空的数据方法
     *
     * @param answerJob
     * @return
     */
    @PostMapping(value = "/save")
    public DataResult save(@RequestBody AnswerJob answerJob) {
        try {
            if (answerJobService.saveNotNull(answerJob) > 0) {
                return DataResult.ok("新增成功");
            } else {
                return DataResult.fail("插入失败");
            }
        } catch (Exception e) {
            log.error("发生错误：{}", e);
            return DataResult.fail(e.getMessage());
        }
    }

    /**
     * 更新AnswerJob属性不为空的数据方法
     *
     * @param answerJob
     * @return
     */
    @PutMapping(value = "/update")
    public DataResult update(@RequestBody AnswerJob answerJob) {
        try {
            if (answerJobService.updateNotNullById(answerJob) > 0) {
                return DataResult.ok("更新成功");
            } else {
                return DataResult.fail("插入失败");
            }
        } catch (Exception e) {
            log.error("发生错误：{}", e);
            return DataResult.fail(e.getMessage());
        }
    }

    /**
     * 通过id删除AnswerJob数据方法
     *
     * @param id
     * @return
     */
    @DeleteMapping(value = "/del/{id}")
    public DataResult delete(@PathVariable(name = "id") Integer id) {
        try {
            if (answerJobService.deleteById(id) > 0) {
                return DataResult.ok("删除成功");
            } else {
                return DataResult.fail("删除失败");
            }
        } catch (Exception e) {
            log.error("发生错误：{}", e);
            return DataResult.fail(e.getMessage());
        }
    }

    public static void main(String[] args) {
        List<Integer> list = new ArrayList<Integer>();
        list.add(1);
        list.add(2);
        list.add(3);
        list.forEach(i -> {
            if(i.intValue() == 1){
                return;
            }
            list.add(null);
            System.out.println(i);
        });
    }
}