package com.web.api.controller.answer.croe;

import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.HttpEntityEnclosingRequest;
import org.apache.http.HttpHost;
import org.apache.http.HttpRequest;
import org.apache.http.NoHttpResponseException;
import org.apache.http.client.HttpRequestRetryHandler;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.protocol.HttpClientContext;
import org.apache.http.config.Registry;
import org.apache.http.config.RegistryBuilder;
import org.apache.http.conn.ConnectTimeoutException;
import org.apache.http.conn.routing.HttpRoute;
import org.apache.http.conn.socket.ConnectionSocketFactory;
import org.apache.http.conn.socket.LayeredConnectionSocketFactory;
import org.apache.http.conn.socket.PlainConnectionSocketFactory;
import org.apache.http.conn.ssl.SSLConnectionSocketFactory;
import org.apache.http.conn.ssl.TrustStrategy;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.impl.conn.DefaultProxyRoutePlanner;
import org.apache.http.impl.conn.PoolingHttpClientConnectionManager;
import org.apache.http.protocol.HttpContext;
import org.apache.http.ssl.SSLContexts;

import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLException;
import javax.net.ssl.SSLHandshakeException;
import java.io.IOException;
import java.io.InterruptedIOException;
import java.net.SocketException;
import java.net.UnknownHostException;
import java.security.KeyManagementException;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;

/**
 * <pre>
 * 名称: HttpRequestConfig
 * 描述: TODO
 * </pre>
 *
 * @author Ryan Chen
 * @since 1.0.0
 */
@Slf4j
public class YtoHttpClientBuilder extends HttpClientBuilder {

    private YtoHttpClientBuilder() {

    }

    public static YtoHttpClientBuilder custom() {
        return new YtoHttpClientBuilder();
    }

    /**
     * create SSLConnectionSocketFactory to trust all
     *
     * @return
     */
    private static SSLConnectionSocketFactory createSSLConnSocketFactory() {
        try {
            SSLContext sslcontext = SSLContexts.custom()
                    .loadTrustMaterial(null, (TrustStrategy) (chain, authType) -> true)
                    .build();
            return new SSLConnectionSocketFactory(sslcontext,
                    SSLConnectionSocketFactory.getDefaultHostnameVerifier());
        } catch (NoSuchAlgorithmException | KeyManagementException | KeyStoreException e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 设置超时
     *
     * @param connectTimeout           连接超时时间
     * @param socketTimeout            等待响应超时
     * @param connectionRequestTimeout 从连接池获取连接超时时间
     * @return
     */
    public YtoHttpClientBuilder timeout(int connectTimeout, int socketTimeout, int connectionRequestTimeout) {

        RequestConfig requestConfig = RequestConfig.custom()
                //从连接池中获取连接的超时时间
                .setConnectionRequestTimeout(connectionRequestTimeout)
                //建立连接超时
                .setConnectTimeout(connectTimeout)
                //等待响应超时（读取数据超时）
                .setSocketTimeout(socketTimeout)
                .build();
        return (YtoHttpClientBuilder) this.setDefaultRequestConfig(requestConfig);
    }

    /**
     * 设置超时
     *
     * @param timeout
     * @return
     */
    public YtoHttpClientBuilder timeout(int timeout) {
        RequestConfig requestConfig = RequestConfig.custom()
                //从连接池中获取连接的超时时间
                .setConnectionRequestTimeout(timeout)
                //建立连接超时
                .setConnectTimeout(timeout)
                //等待响应超时（读取数据超时）
                .setSocketTimeout(timeout)
                .build();
        return (YtoHttpClientBuilder) this.setDefaultRequestConfig(requestConfig);
    }

    /**
     * 设置连接池
     *
     * @param maxTotal    最大连接数
     * @param maxPerRoute 每个路由的最大连接数
     * @param maxRoute    最大路由数据
     * @param hostname    主机
     * @param port        端口
     * @return
     */
    public YtoHttpClientBuilder pool(int maxTotal, int maxPerRoute, int maxRoute, String hostname, int port) {
        ConnectionSocketFactory plainsf = PlainConnectionSocketFactory
                .getSocketFactory();
        LayeredConnectionSocketFactory sslsf = SSLConnectionSocketFactory
                .getSocketFactory();
        Registry<ConnectionSocketFactory> registry = RegistryBuilder
                .<ConnectionSocketFactory>create().register("http", plainsf)
                .register("https", sslsf).build();
        PoolingHttpClientConnectionManager cm = new PoolingHttpClientConnectionManager(
                registry);
        //每隔 2 秒才检查连接的有效性
        cm.setValidateAfterInactivity(2000);
        // 将最大连接数增加
        cm.setMaxTotal(maxTotal);
        // 将每个路由基础的连接增加
        cm.setDefaultMaxPerRoute(maxPerRoute);
        HttpHost httpHost = new HttpHost(hostname, port);
        // 将目标主机的最大连接数增加
        cm.setMaxPerRoute(new HttpRoute(httpHost), maxRoute);
        return (YtoHttpClientBuilder) this.setConnectionManager(cm);
    }

    /**
     * 设置连接池
     *
     * @param maxTotal    最大连接数
     * @param maxPerRoute 每个路由的最大连接数
     * @return
     */
    public YtoHttpClientBuilder pool(int maxTotal, int maxPerRoute) {
        ConnectionSocketFactory plainsf = PlainConnectionSocketFactory
                .getSocketFactory();
        LayeredConnectionSocketFactory sslsf = SSLConnectionSocketFactory
                .getSocketFactory();
        Registry<ConnectionSocketFactory> registry = RegistryBuilder
                .<ConnectionSocketFactory>create().register("http", plainsf)
                .register("https", sslsf).build();
        PoolingHttpClientConnectionManager cm = new PoolingHttpClientConnectionManager(
                registry);
        // 将最大连接数增加
        cm.setMaxTotal(maxTotal);
        // 将每个路由基础的连接增加
        cm.setDefaultMaxPerRoute(maxPerRoute);

        return (YtoHttpClientBuilder) this.setConnectionManager(cm);
    }

    /**
     * 重试（如果请求是幂等的，就再次尝试）
     *
     * @param tryTimes               重试次数
     * @param retryWhenInterruptedIO 连接拒绝时，是否重试
     * @return 返回当前对象
     */
    public YtoHttpClientBuilder retry(final int tryTimes, final boolean retryWhenInterruptedIO) {
        // 请求重试处理
        HttpRequestRetryHandler httpRequestRetryHandler = new HttpRequestRetryHandler() {

            @SneakyThrows
            @Override
            public boolean retryRequest(IOException exception, int executionCount, HttpContext context) {
                // 如果已经重试了n次，就放弃
                if (executionCount > tryTimes) {
                    log.error("已重试{}次，放弃本次请求",tryTimes);
                    return false;
                }
                // 如果服务器丢掉了连接，那么就重试
                if (exception instanceof NoHttpResponseException) {
                    return true;
                }
                // 不要重试SSL握手异常
                if (exception instanceof SSLHandshakeException) {
                    return false;
                }
                // 超时重连
                if (exception instanceof InterruptedIOException) {
                    log.error("连接超时，开始第{}次重连……",executionCount);
                    Thread.sleep(10000);
                    return retryWhenInterruptedIO;
                }
                // 目标服务器不可达
                if (exception instanceof UnknownHostException) {
                    log.error("目标服务器不可达");
                    return true;
                }
                // 连接被拒绝
                if (exception instanceof ConnectTimeoutException) {
                    return false;
                }
                // SSL握手异常
                if (exception instanceof SSLException) {
                    return false;
                }
                //连接被重置
                if (exception instanceof SocketException) {
                    log.error("连接被重置，开始第{}次重连……",executionCount);
                    Thread.sleep(10000);
                    return retryWhenInterruptedIO;
                }
                HttpClientContext clientContext = HttpClientContext.adapt(context);
                HttpRequest request = clientContext.getRequest();
                // 如果请求是幂等的，就再次尝试
                if (!(request instanceof HttpEntityEnclosingRequest)) {
                    return true;
                }
                return false;
            }
        };
        return (YtoHttpClientBuilder) this.setRetryHandler(httpRequestRetryHandler);
    }

    /**
     * 重试
     *
     * @param tryTimes 重试次数
     * @return
     */
    public YtoHttpClientBuilder retry(final int tryTimes) {
        return retry(tryTimes, false);
    }

    /**
     * 设置代理
     *
     * @param hostOrIP 代理host或者ip
     * @param port     代理端口
     * @return 返回当前对象
     */
    public YtoHttpClientBuilder proxy(String hostOrIP, int port) {
        // 依次是代理地址，代理端口号，协议类型
        HttpHost proxy = new HttpHost(hostOrIP, port, "http");
        DefaultProxyRoutePlanner routePlanner = new DefaultProxyRoutePlanner(proxy);
        return (YtoHttpClientBuilder) this.setRoutePlanner(routePlanner);
    }

    /**
     * 设置ssl安全链接
     *
     * @return 返回当前对象
     */
    public YtoHttpClientBuilder ssl() {
        return (YtoHttpClientBuilder) this.setSSLSocketFactory(createSSLConnSocketFactory());
    }
}