package com.web.api.service.impl;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.support.ExcelTypeEnum;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.web.api.controller.answer.croe.HttpRequestConfig;
import com.web.api.controller.answer.croe.HttpRequestResult;
import com.web.api.dao.AnswerDao;
import com.web.api.exception.BusinessException;
import com.web.api.exception.ErrorCode;
import com.web.api.exception.SgException;
import com.web.api.listener.ImportExcelListener;
import com.web.api.model.dto.AnswerDto;
import com.web.api.model.dto.SearchDto;
import com.web.api.model.po.AnswerExcel;
import com.web.api.model.po.AnswerRecord;
import com.web.api.model.po.UserInfo;
import com.web.api.model.po.SgEntity;
import com.web.api.service.AnswerService;
import com.web.api.service.mail.MailServiceImpl;
import com.web.api.utils.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.RandomUtils;
import org.apache.http.client.CookieStore;
import org.apache.http.client.protocol.HttpClientContext;
import org.apache.http.protocol.HttpContext;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.EnableAsync;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Method;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.text.DecimalFormat;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.Base64;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.regex.Matcher;

import static com.web.api.utils.Constant.MOBILE_API_URL;
import static com.web.api.utils.Constant.SERVER_IP;


/**
 * @author 王玉鹏
 * @version 1.0
 * @className AnswerServiceImpl
 * @description TODO
 * @date 2020/7/8 14:11
 */
@Service
@Slf4j
public class AnswerServiceImpl implements AnswerService {


    @Resource
    private AnswerDao answerDao;

    @Autowired
    private AnswerManager answerManager;

    @Resource
    private ThreadPoolTaskExecutor asyncServiceExecutor;

    @Autowired
    private MailServiceImpl mailService;

    @Autowired
    private EasyExcelUtil easyExcelUtil;

    @Autowired
    private OssClientUtils ossClientUtils;

    @Value("${upload.file.tempDir}")
    private String tempDir;

    @Value("${upload.file.fileDir}")
    private String fileDir;

    @Value("${sgAnswer.admin.jobNo}")
    private String adminJobNo;

    @Value("${sgAnswer.admin.password}")
    private String adminPassWord;

    private static final Map<String, CookieStore> standAdminMap = new ConcurrentHashMap<>();
    private static final Map<String, Map<String, Object>> userParam = new ConcurrentHashMap<>();
    private String currentAdminJobNo;



    @Override
    public CookieStore login(String username, String password, String safeCode) throws Exception {
        String safeCodeData = answerDao.getSafeCode();
        log.info("当前系统安全码为：{}，接口参数安全码为：{}", safeCodeData, safeCode);
        if (StringUtils.isEmpty(safeCode) || !safeCodeData.equals(safeCode)) {
            throw new SgException(ErrorCode.SAFE_CODE_ERROR.getCode(), ErrorCode.SAFE_CODE_ERROR.getName());
        }
        if (!ObjectUtils.isEmpty(ContextUtils.getUserCookies(username)) && ContextUtils.getUserCookies(username).getCookies().size() > 1) {
            log.info("当前登陆用户:【{}】,cookie已存在，已从上下文中获取", username);
            return ContextUtils.getUserCookies(username);
        }
        log.info("当前登陆用户:【{}】,调用第三方接口获取cookie", username);

        Map<String, String> map = new HashMap<>(10);
        map.put("username", username);
        map.put("password", MD5Util.encryptPassword(username,password));
        CookieStore cookie = CookieUtil.getCookies(MOBILE_API_URL + "users/chklogin", map, "UTF-8");
        if (ObjectUtils.isEmpty(cookie) || cookie.getCookies().size() <= 1) {
            log.info("cookie为空,登陆失败");
            throw new SgException(ErrorCode.PASSWORD_ERROR.getCode(), ErrorCode.PASSWORD_ERROR.getName());
        }
        ContextUtils.setUserCookies(username, cookie);
        return cookie;
    }

    @Override
    public UserInfo getUserInfo(String jobNo, CookieStore cookieStore) throws Exception {
        checkCookie(cookieStore);
        ServletRequestAttributes servletRequestAttributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = servletRequestAttributes.getRequest();
        log.info("当前客户端登陆ip:{}", SgUtils.getIpAddr(request));
        UserInfo user = getLatestUserInfo(cookieStore);
        user.setJobNo(jobNo);
        user.setTargetScore(ContextUtils.getUserInfo(jobNo) == null || null == ContextUtils.getUserInfo(jobNo).getTargetScore() ? 0 : ContextUtils.getUserInfo(jobNo).getTargetScore());
        //获取最新用户信息，如果上下文没有该用户信息，则放入上下文
        if (ObjectUtils.isEmpty(ContextUtils.getUserInfo(jobNo))) {
            ContextUtils.setUserInfo(jobNo, user);
        } else {
            ContextUtils.getUserInfo(jobNo).setIntegral(user.getIntegral());
        }
        ContextUtils.getUserInfo(jobNo).setCurrentIp(SgUtils.getIpAddr(request));
        //开启线程获客户端mac
//        Thread t = new Thread( new GetClientMac(SgUtils.getIpAddr(request)));
//        t.start();
        if (StringUtils.isEmpty(user.getUserName())) {
            log.error("cookie已失效，移除本地用户：{}cookie", jobNo);
            ContextUtils.removeUserCookies(jobNo);
            ContextUtils.removeUserInfo(jobNo);
            throw new SgException(ErrorCode.LOGOUT.getCode(), ErrorCode.LOGOUT.getName());
        }
        //绑定当前登陆用户
        ContextUtils.setUsers(user);
        return user;

    }

    @Override
    public void answer(CookieStore cookieStore, AnswerDto answerDto) throws Exception {
        checkCookie(cookieStore);
        ContextUtils.setStatus(true);
        UserInfo userInfo = getUser(answerDto.getJobNo(), cookieStore);
        userInfo.setErrMsg(null);
        userInfo.setAnswerStatus(answerDto.getBatchFlag() ? "批量答题中" : "答题中");
        userInfo.setAddScores("0");
        userInfo.setTargetScore(answerDto.getTargetScore());
        userInfo.setJobType(answerDto.getJobTypeId());
        userInfo.setDifficultyLevel(answerDto.getLevel());
        userInfo.setJobTypeName(answerDto.getJobTypeName());
        DateTimeFormatter fmt = DateTimeFormatter.ofPattern("yyyy年MM月dd日 HH时mm分");
        userInfo.setStartTime(LocalDateTime.now().format(fmt));
        ContextUtils.setOnOff(answerDto.getJobNo(), true);

        SgEntity sgEntity = new SgEntity();

        int currentGs = AnswerManager.getCurrentLevel(cookieStore, answerDto.getLevel(), answerDto.getJobTypeId());

        sgEntity.setCurrentGs(currentGs);
        sgEntity.setDifficultyLevel(answerDto.getLevel());
        sgEntity.setJobTypeId(answerDto.getJobTypeId());

        final Double startScore = userInfo.getIntegral();
        log.info("用户{}开始准备答题……，当前活动线程总数量:【{}】", answerDto.getJobNo(), Thread.activeCount());
        log.info("当前异步线程池的线程数量:【{}】, 当前异步线程池活动线程数量 【{}】",
                asyncServiceExecutor.getPoolSize(),
                asyncServiceExecutor.getActiveCount());
        //批量答题处理逻辑
        if (answerDto.getBatchFlag()) {
            userInfo.setJobNo(answerDto.getJobNo());
            ContextUtils.setUserInfo(answerDto.getJobNo(), userInfo);
            ContextUtils.setBatchUserList(answerDto.getJobNo(), userInfo);
        }
        answerManager.anwser(cookieStore, sgEntity, answerDto, startScore);
        int res = answerDao.save(userInfo, answerDto);
        if (res > 0) {
            log.info("已保存本次答题记录初始分数:{}", userInfo);
        }
    }

    @Override
    public Map<String, Object> getAnwserInfo(String jobNo, CookieStore cookieStore) throws Exception {
        log.info("工号====================:{}", jobNo);
        checkCookie(cookieStore);
        Map<String, Object> lastRecord = answerDao.queryAnwserInfo(jobNo);
        UserInfo userInfo = ContextUtils.getUserInfo(jobNo);
        log.info("查询到当前用户最新记录为:{}", lastRecord);
        lastRecord.put("errorMsg", userInfo.getErrMsg());
        if (!StringUtils.isEmpty(userInfo.getErrMsg()) && !userInfo.getErrMsg().contains("上限")) {
            lastRecord.put("latestScores", userInfo.getIntegral());
            return lastRecord;
        }
        UserInfo latestUserInfo = getLatestUserInfo(cookieStore);
        DecimalFormat df = new DecimalFormat("######0.00");
        if (ObjectUtils.isEmpty(lastRecord)) {
            lastRecord = new HashMap<>(10);
        } else {
            lastRecord.put("addScores", df.format(latestUserInfo.getIntegral() - Double.parseDouble(lastRecord.get("startScore").toString())));
        }
        //默认查询当天答题结束记录
        LocalDateTime today_start = LocalDateTime.of(LocalDate.now(), LocalTime.MIN);
        LocalDateTime today_end = LocalDateTime.of(LocalDate.now(), LocalTime.MAX);
        DateTimeFormatter time = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        String todayStart = time.format(today_start);
        String todayEnd = time.format(today_end);
        List<AnswerRecord> records = answerDao.queryAnwserRecord(todayStart, todayEnd);
        lastRecord.put("latestScores", df.format(latestUserInfo.getIntegral()));
        lastRecord.put("latestGs", userInfo.getCurrentGk());
        lastRecord.put("jobTypeId", userInfo.getJobType());
        lastRecord.put("difficultyLevel", userInfo.getDifficultyLevel());
        lastRecord.put("currentJobType", userInfo.getJobTypeName());
        lastRecord.put("onOff", ContextUtils.getOnOff(jobNo));
        lastRecord.put("records", records);
        return lastRecord;
    }

    @Override
    public void stopAnswer(String jobNo, CookieStore cookieStore) throws Exception {
        //答题状态置为false
        log.info("用户：【{}】 发起停止答题请求", jobNo);
        ContextUtils.setOnOff(jobNo, false);
    }

    @Override
    public void logOut(String jobNo, CookieStore cookieStore) throws Exception {
        //退出时删除未答题用户登陆对象（只保留云答题用户）
        UserInfo userInfo = getUser(jobNo, cookieStore);
        if (StringUtils.isEmpty(userInfo.getAnswerStatus()) || !userInfo.getAnswerStatus().equals("答题中")) {
            log.info("闲置用户：【{}】 退出登陆", jobNo);
            ContextUtils.removeUserCookies(jobNo);
            ContextUtils.removeUserInfo(jobNo);
        }

    }

    @Override
    public String ranking(String jobNo, CookieStore cookieStore) throws Exception {
        String res = SgUtils.sgRequest("get", cookieStore, "phb/personal");

        return res.replaceAll("/mobilestyles/", "http://117.39.28.234:8023/mobilestyles/").
                replace("更多", "").replace("点击查看...", "只展示前20排名").
                replace("<span class=\"aui-iconfont aui-icon-left\"></span>", "<span class=\"mint-button-icon\"><i class=\"mintui mintui-back\"></i></span>");
    }

    @Override
    public String exportExcel(SearchDto searchDto) throws Exception {
        log.info("开始生成excel中……");
        String startDateTime;
        String endDateTime;
        String fileName;
        //默认当天记录
        if (StringUtils.isEmpty(searchDto.getStartDate()) || StringUtils.isEmpty(searchDto.getEndDate())) {
            LocalDateTime today_start = LocalDateTime.of(LocalDate.now(), LocalTime.MIN);
            LocalDateTime today_end = LocalDateTime.of(LocalDate.now(), LocalTime.MAX);
            DateTimeFormatter time = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
            startDateTime = time.format(today_start);
            endDateTime = time.format(today_end);
            fileName = "todayRecords" + LocalDate.now(ZoneId.of("Asia/Shanghai")) + ExcelTypeEnum.XLSX.getValue();
        } else {
            startDateTime = searchDto.getStartDate() + " 00:00:00";
            endDateTime = searchDto.getEndDate() + " 23:59:59";
            fileName = "answerRecords" + searchDto.getStartDate() + "to" + searchDto.getEndDate() + ExcelTypeEnum.XLSX.getValue();
        }
        List<AnswerRecord> records = answerDao.queryAnwserRecord(startDateTime, endDateTime);
        File file = new File(tempDir);
        if (!file.exists()) {
            file.mkdirs();//创建目录
        }
        String filePath = tempDir + fileName;
        easyExcelUtil.writeExcel(records, filePath, "answerRecords", AnswerRecord.class);
        log.info("生成excel成功，开始上传oss……");
        InputStream inputStream = new FileInputStream(filePath);
        String keyName = ossClientUtils.uploadFile2OSS(inputStream, fileName, fileDir);
        String fileUrl = ossClientUtils.getUrl(fileName);
        log.info("上传文件：{}至阿里oss成功,url:{}", keyName, fileUrl);
        return fileUrl;
    }

    @Override
    public List<AnswerRecord> search(SearchDto searchDto) {
        String startDateTime = searchDto.getStartDate() + " 00:00:00";
        String endDateTime = searchDto.getEndDate() + " 23:59:59";
        return answerDao.queryAnwserRecord(startDateTime, endDateTime);
    }

    @Override
    public String importExcel(MultipartFile file) throws IOException {
        if (file == null || file.getSize() <= 0) {
            log.info("发生错误{}", "文件为空");
            throw new BusinessException("不能上传空文件");
        }
        if (!file.getOriginalFilename().endsWith(".xls") && !file.getOriginalFilename().endsWith(".xlsx")) {
            throw new BusinessException("文件格式不正确，请上传xls或xlsx的文件");
        }
        //每次导入批量答题前，清空批量用户列表
        ContextUtils.clearBatchUserList();
        log.info("文件格式校验通过，开始调用第三方接口答题");
        EasyExcel.read(file.getInputStream(), AnswerExcel.class, new ImportExcelListener(this, mailService)).sheet().doRead();
        return "excel导入成功，开始批量答题";
    }

    @Override
    public String updateTime(int time) throws Exception {
        Map<String, Object> paramMap = new HashMap<>();
        paramMap.put("title", "陕钢集团智慧工会");
        paramMap.put("group_name", "陕钢公司工会");
        paramMap.put("systemuserid", "30479");
        paramMap.put("realname", "管理员");
        paramMap.put("telephone", "13280520213");
        paramMap.put("bmzw", "工作人员");
        paramMap.put("email", "");
        paramMap.put("stzjjf", "10");
        paramMap.put("zjdyjf", "");
        paramMap.put("loginjf", "10");
        paramMap.put("eggnum", "20");
        paramMap.put("timelimit", time);
        paramMap.put("clickrate", "1535224");
        paramMap.put("qyid", "1");
        HttpContext httpContext = new HttpClientContext();
        ((HttpClientContext) httpContext).setCookieStore(getAdminCookie());
        getAdminIndex(httpContext);
        HttpRequestConfig config = HttpRequestConfig.create().url(SERVER_IP+"/admin/company/systemset").context(httpContext).paramMap(paramMap);
        HttpRequestResult result = HttpClientUtils.post(config);
        String data = result.getResponseText();
        log.info("操作结果：{}", data);
        return data;
    }

    @Override
    public Object getLimitTime() throws Exception {
        //获取第三方时间管理页面
        HttpContext httpContext = new HttpClientContext();
        ((HttpClientContext) httpContext).setCookieStore(getAdminCookie());
        getAdminIndex(httpContext);
        HttpRequestConfig config = HttpRequestConfig.create().url(SERVER_IP+"/admin/company/systemset").context(httpContext);
        HttpRequestResult result = HttpClientUtils.post(config);
        String data = result.getResponseText();
        System.out.println(data);
        String methodName = new Throwable().getStackTrace()[0].getMethodName();
        log.info("currentMethodName:{}", methodName);
        Object obj = invoke(methodName,data);
        if (!ObjectUtils.isEmpty(obj)){
            return obj;
        }
        //获取答题上限时间
        Matcher m3 = Constant.TIME_LIMIT_PATTENT.matcher(data);
        String timeLimit = "";
        while (m3.find()) {
            timeLimit = m3.group(2);
        }
        log.info("timeLimit:{}", timeLimit);
        return timeLimit;
    }

    @Override
    public Object getForbidden(String jobNo) throws Exception {
        //获取管理员cookie
        HttpContext httpContext = new HttpClientContext();
        ((HttpClientContext) httpContext).setCookieStore(getAdminCookie());
//        getAdminIndex(httpContext);
        //管理员权限根据工号获取用户基本信息
        HttpRequestConfig config = HttpRequestConfig.create().url(SERVER_IP+"/admin/users?groupaid=&groupbid=&keyword=" + jobNo).context(httpContext);
        HttpRequestResult result = HttpClientUtils.post(config);
        String data = result.getResponseText();

        //如果cookie失效则重新获取cookie进行此操作
        String methodName = new Throwable().getStackTrace()[0].getMethodName();
        log.info("currentMethodName:{}", methodName);
        Object obj = invoke(methodName,data,jobNo);
        if (!ObjectUtils.isEmpty(obj)){
            return obj;
        }
        //提取列表userId字段
        Matcher userIdPattent = Constant.USER_ID_PATTENT.matcher(data);
        String userId = "";
        while (userIdPattent.find()) {
            userId = userIdPattent.group(2);
        }
        if (data.contains("Connection")){
            log.error("查询【{}】连接异常，尝试递归调用",jobNo);
            getAdminIndex(httpContext);
            return getForbidden(jobNo);
        }
        if (StringUtils.isEmpty(userId)){
            log.error("未获取到【{}】该用户id,结果响应：{}", jobNo, data);
            throw new Exception("未获取到【"+jobNo+"】用户信息");
        }
        System.out.println("userId:" + userId);
        //管理员权限根据userId获取用户详情
        HttpRequestConfig userCon = HttpRequestConfig.create().url(SERVER_IP+"/admin/users/info?id=" + userId).context(httpContext);
        HttpRequestResult userRes = HttpClientUtils.get(userCon);
        String useText = userRes.getResponseText();
//        System.out.println("useText:"+useText);
        //提取表单input标签字段
        Matcher userFiledPattent = Constant.USER_FIELD.matcher(useText);
        //提取表单select标签字段
        Matcher userSelectPattent = Constant.USER_SELECT_PATTENT.matcher(useText);
        Map<String, Object> map = new HashMap<>();
        Boolean status = false;
        while (userFiledPattent.find()) {
            //提取用户禁用状态标签
            if (StringUtils.isEmpty(userFiledPattent.group(2))) {
                System.out.println("用户禁用状态:" + userFiledPattent.group(0));
                if (userFiledPattent.group(0).contains("checked")) {
                    status = false;
                } else {
                    status = true;
                }
            } else {
                map.put(userFiledPattent.group(2), userFiledPattent.group(4));
                System.out.println(userFiledPattent.group(2) + "===========" + userFiledPattent.group(4));
            }
        }
        while (userSelectPattent.find()) {
            map.put(userSelectPattent.group(1), userSelectPattent.group(2));
            System.out.println(userSelectPattent.group(1) + "============" + userSelectPattent.group(2) + "==========" + userSelectPattent.group(3));
        }
        //处理性别转换
        if (!map.get("gender").toString().equals("0") && !map.get("gender").toString().equals("1")) {
            map.put("groupaid", map.get("gender"));
            map.put("gender", 1);
        }
        map.put("userId", userId);
        map.put("status", status);
        log.info("paramMap:{}", map);
        userParam.put(jobNo, map);
        return map;
    }

    @Override
    public boolean updateForbiddenStatus(String jobNo, Boolean status,int bus) throws Exception {
        //根据工号获取管理cookie
        HttpContext httpContext = new HttpClientContext();
        ((HttpClientContext) httpContext).setCookieStore(getAdminCookie());
        Map<String, Object> paramMap = userParam.get(jobNo);
        log.info("传入参数:{},{}", jobNo, status);
        //禁用账户
        if (!status) {
            paramMap.put("status", "Y");
        }else {
            paramMap.remove("status");

        }
        //更新用户信息
        log.info("update paramMap:{}", paramMap);
        HttpRequestConfig updateConf = HttpRequestConfig.create().url(SERVER_IP+"/admin/users/info?id=" + userParam.get(jobNo).get("userId")).context(httpContext).paramMap(paramMap);
        HttpRequestResult upRes = HttpClientUtils.post(updateConf);
        String updateText = upRes.getResponseText();
        System.out.println(updateText);
        if (updateText.contains("操作成功")){
            if (bus == 1){
                log.info("冻结成功，开始进行本地DB操作……");
                new Thread(() -> updateBackList(jobNo,status,paramMap.get("realname").toString())).start();
            }
            return true;
        }else {
            return false;
        }
    }

    public void updateBackList(String jobNo,Boolean status,String realname){
        int isOpen = !status?1:0;
        Map<String,Object> paramMap = new HashMap<>();
        paramMap.put("isOpen",isOpen);
        paramMap.put("jobNo",jobNo);
        paramMap.put("userName",realname);
        List<Map<String,Object>> list = answerDao.queryBlacklist(jobNo);
        if (list.size()>0){
            log.info("系统黑名单已存在【{}】用户，更新job开启状态为：{}",jobNo,isOpen);
            list.get(0).put("isOpen",isOpen);
            answerDao.updateBlackList(list.get(0));
        }else {
            log.info("将用户【{}】加入自动封禁队列",jobNo);
            answerDao.insertBlackList(paramMap);
        }
    }

    @Override
    public boolean resetPassword(String jobNo) throws Exception {
        //根据工号获取管理cookie
        HttpContext httpContext = new HttpClientContext();
        ((HttpClientContext) httpContext).setCookieStore(getAdminCookie());
        Map<String, Object> paramMap = new HashMap<>();
        if (ObjectUtils.isEmpty(userParam.get(jobNo))){
            throw new SgException(ErrorCode.PARAM_EMPTY.getCode(), ErrorCode.PARAM_EMPTY.getName());
        }
        //获取用户id
        String userId = userParam.get(jobNo).get("userId").toString();
        log.info("工号:{}，userId:{}", jobNo,userId);
        HttpRequestConfig resConf = HttpRequestConfig.create().url(SERVER_IP+"/admin/users/reduction?id=" + userId).context(httpContext).paramMap(paramMap);
        HttpRequestResult res = HttpClientUtils.post(resConf);
        String resText = SgUtils.unicodeToString(res.getResponseText()) ;
        log.info(resText);
        JSONObject jsonObject = JSON.parseObject(resText);
        if (resText.contains("初始化密码成功")){
            return true;
        }else {
            throw new SgException(9999, jsonObject.getString("msg"));
        }
    }

    private void checkCookie(CookieStore cookie) throws Exception {
        if (ObjectUtils.isEmpty(cookie) || cookie.getCookies().size() <= 1) {
            log.info("cookie为空,登陆已过期");
            throw new SgException(ErrorCode.COOKIE_PAST_DUE.getCode(), ErrorCode.COOKIE_PAST_DUE.getName());
        }
    }

    private UserInfo getLatestUserInfo(CookieStore cookieStore) throws Exception {
        String data = SgUtils.sgRequest("post", cookieStore, "users");
        UserInfo latestUserInfo = AnswerManager.makeUserInfo(data);
        if (StringUtils.isEmpty(latestUserInfo.getUserName())){
            throw new SgException(ErrorCode.USERS_EMPTY.getCode(), ErrorCode.USERS_EMPTY.getName());
        }
        return latestUserInfo;
    }

    /**
     * 从缓存中获取当前用户信息，如果不存在则取第三方接口拉去最新用户信息放入缓存
     */
    private UserInfo getUser(String JobNo, CookieStore cookieStore) throws Exception {
        if (ObjectUtils.isEmpty(ContextUtils.getUserInfo(JobNo))) {
            ContextUtils.setUserInfo(JobNo, getLatestUserInfo(cookieStore));
            return getLatestUserInfo(cookieStore);
        } else {
            return ContextUtils.getUserInfo(JobNo);
        }
    }

    /**
     * 获取管理员cookie
     */
    private CookieStore getAdminCookie() throws SgException {
        //随机取管理员账号
        String[] jobs = adminJobNo.split(",");
        int index = RandomUtils.nextInt(0,jobs.length);
        String job = jobs[index];
        currentAdminJobNo = job;
        //如账号存在则取缓存中cookie
        if (standAdminMap.containsKey(currentAdminJobNo)) {
            log.info("管理员账户【{}】已存在,直接获取",currentAdminJobNo);
            if (ObjectUtils.isEmpty(standAdminMap.get(currentAdminJobNo)) || standAdminMap.get(currentAdminJobNo).getCookies().size() <= 1) {
                log.info("管理员账户【{}】已在其他地方登錄,重新獲取token",currentAdminJobNo);
            } else {
                return standAdminMap.get(currentAdminJobNo);
            }
        }
        //调用第三方接口获取管理员cookie
        Map<String, String> map = new HashMap<>(15);
        map.put("username", currentAdminJobNo);
        map.put("password",  MD5Util.encryptPassword(currentAdminJobNo,adminPassWord));
        log.info("登录请求参数:{}",JSON.toJSONString(map));
        CookieStore cookie = CookieUtil.getCookies(SERVER_IP+"/index/users/chklogin", map, "UTF-8");
        if (ObjectUtils.isEmpty(cookie) || cookie.getCookies().size() <= 1) {
            log.info("登录管理员账【{}】户失败", currentAdminJobNo);
        }
        standAdminMap.put(currentAdminJobNo, cookie);
        return cookie;
    }
    /**
     * 管理员后台机制验证，需先打开用户页面，再打开管理员首页，才能进行管理员功能操作
     */
    private void getAdminIndex(HttpContext httpContext) throws Exception {
        HttpRequestConfig configC = HttpRequestConfig.create().url(SERVER_IP+"/index/index/sgindex").context(httpContext);
        HttpRequestResult resultC = HttpClientUtils.get(configC);
//        log.info("resultC:{}",resultC.getResponseText());
        HttpRequestConfig configS = HttpRequestConfig.create().url(SERVER_IP+"/admin").context(httpContext);
        HttpRequestResult resultS = HttpClientUtils.get(configS);
//        log.info("resultC:{}",resultS.getResponseText());
    }

    @Override
    public void clearAdminCookie(){
        log.info("清空管理员登录信息");
        standAdminMap.clear();
    }
    /**
     * @author wangyupeng
     * @description cookie失效删除cookie递归重试
     * @date  2021/12/15 19:59
     * @param methodName 方法名
     * @param args  data：第三方返回结果，jobNo：工号
     * @return java.lang.Object
     */
    private Object invoke(String methodName,String... args) throws Exception {
       if (StringUtils.isEmpty(args[0]) || args[0].contains("重新登录")) {
           log.info("cookie已失效:{},{}", methodName,args);
           standAdminMap.remove(adminJobNo);
           Object res;
           if (args.length>1){
               Method m = this.getClass().getMethod(methodName,String.class);
               res=m.invoke(this,args[1]);
           }else {
               Method m = this.getClass().getMethod(methodName);
               res= m.invoke(this);
           }
           return res;
       }
       return null;
    }


     @Override
     public List<Map<String,Object>> queryBlacklist(){
      return answerDao.queryBlacklist(null);
    }

    //    public static void main(String[] args) {
//        String fileName ="ssddhsg.xlsx";
//        System.out.println(fileName.substring(fileName.lastIndexOf(".")).toLowerCase());
//    }
//    public static void main(String[] args) {
//        Map<String, Object> map = new HashMap<>();
//        map.put("time", 1);
//        HttpRequestConfig config = HttpRequestConfig.create().url("http://localhost:8888/answer/api/updateTime").paramMap(map);
//        HttpRequestResult result = null;
//        try {
//            result = HttpClientUtils.post(config);
//        } catch (Exception e) {
//            e.printStackTrace();
//        }
//        System.out.println(result.getResponseText());
//    }
}
