package com.web.api.utils;

import com.google.common.collect.Maps;
import com.web.api.model.po.UserInfo;
import org.apache.http.client.CookieStore;


import java.util.Map;
import java.util.concurrent.atomic.AtomicBoolean;

/**
 * @author 王玉鹏
 * @version 1.0
 * @className ContextUtils
 * @description  上下文处理类
 * @date 2020/7/10 9:38
 */
public class ContextUtils {


     private static ThreadLocal<UserInfo> users ;
    /**
     * 用户基本信息
     */
    private static Map<String, UserInfo> usersInfo = Maps.newConcurrentMap();
    /**
     * 用户答题中/答题结束识别开关
     */
    private static Map<String, Boolean> onOff = Maps.newConcurrentMap();
    /**
     * 用户本地请求第三方接口cookie
     */
    private static Map<String, CookieStore> userCookies = Maps.newConcurrentMap();
    /**
     * 用户本地请求第三方接口cookieString
     */
    private static String userStringCookie = "";
    /**
     * 一键终止答题标识
     */
    private static  AtomicBoolean status = new AtomicBoolean(true);
    /**
     * 批量答题用户集合
     */
    private static Map<String, UserInfo> batchUsersInfo = Maps.newConcurrentMap();

//    /**用户当前答题记录关卡*/
//    public static Map<String, Integer> gs = Maps.newConcurrentMap();
//    /**用户答题异常信息*/
//    public static Map<String, String> errMsg = Maps.newConcurrentMap();
//    /**用户数据库记录id*/
//    public static Map<String, Integer> ids = Maps.newConcurrentMap();
//    /**用户答题目标分数id*/
//    public static Map<String, Integer> targetScores = Maps.newConcurrentMap();

    public static void setUserInfo(String jobNo, UserInfo user) {
        ContextUtils.usersInfo.put(jobNo, user);
    }

    public static UserInfo getUserInfo(String jobNo) {
        return ContextUtils.usersInfo.get(jobNo);
    }

    public static void removeUserInfo(String jobNo) {
        ContextUtils.usersInfo.remove(jobNo);
    }

    public static CookieStore getUserCookies(String jobNo) {
        return ContextUtils.userCookies.get(jobNo);
    }

    public static void setUserCookies(String jobNo, CookieStore cookieStore) {
        ContextUtils.userCookies.put(jobNo, cookieStore);
    }

    public static void removeUserCookies(String jobNo) {
        ContextUtils.userCookies.remove(jobNo);
    }

    public static Boolean getOnOff(String jobNo) {
        return onOff.get(jobNo);
    }

    public static void setOnOff(String jobNo, Boolean onOff) {
        ContextUtils.onOff.put(jobNo, onOff);
    }

    public static Map getUserCookie() {
        return userCookies;
    }

    public static Map getOnlineUsers() {
        return usersInfo;
    }

    public static ThreadLocal<UserInfo> getUsers() {
        return users;
    }

    public static void setUsers(UserInfo users) {
        ThreadLocal<UserInfo> currentUser = new ThreadLocal<>();
        currentUser.set(users);
        ContextUtils.users = currentUser;
    }

    public static String getUserStringCookie() {
        return userStringCookie;
    }

    public static void setUserStringCookie(String userStringCookie) {
        ContextUtils.userStringCookie = userStringCookie;
    }

    public static Boolean getStatus() {
        return status.get();
    }

    public static void setStatus(Boolean status) {
        ContextUtils.status = new AtomicBoolean(status);
    }

    public static Map getBatchUserList() {
        return batchUsersInfo;
    }
    public static void setBatchUserList(String jobNo,UserInfo userInfo) {
        ContextUtils.batchUsersInfo.put(jobNo, userInfo);
    }
    public static void clearBatchUserList() {
        ContextUtils.batchUsersInfo.clear();
    }


}
