package com.web.api.utils;


import com.web.api.controller.answer.croe.HttpRequestConfig;
import com.web.api.controller.answer.croe.HttpRequestMethod;
import com.web.api.controller.answer.croe.HttpRequestResult;
import com.web.api.controller.answer.croe.YtoHttpClientBuilder;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpEntity;
import org.apache.http.HttpHost;
import org.apache.http.HttpResponse;
import org.apache.http.HttpVersion;
import org.apache.http.client.methods.*;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.protocol.HTTP;
import org.apache.http.util.EntityUtils;

import java.io.IOException;
import java.io.OutputStream;

/**
 * <pre>
 * 名称: HttpClientUtils
 * 描述: TODO
 * </pre>
 *
 * @author
 * @since 1.0.0
 */
@Slf4j
public class HttpClientUtils {

    private final static Object syncLock = new Object();
    private static final int CONNECT_TIMEOUT = 60000;
    private static final int SOCKET_TIMEOUT = 60000;
    private static final int CONNECT_REQUEST_TIMEOUT = 10000;
    private static final int MAX_TOTAL = 1000;
    private static final int MAX_PER_ROUTE = 200;
    private static final int RETRY_TIMES = 10;
    /**
     * 默认采用的http协议的HttpClient对象
     */
    private static CloseableHttpClient httpClient;
    /**
     * 默认采用的https协议的HttpClient对象
     */
    private static CloseableHttpClient httpsClient;

    /**
     * 初始化 httpCient
     *
     * @param httpRequestConfig
     */
    private static void setHttpClinet(HttpRequestConfig httpRequestConfig) {
//        HttpHost proxy = new HttpHost("101.43.173.238",8888,"http");
        if (httpRequestConfig.getHttpClient() == null) {
            if (httpRequestConfig.isUseSSL()) {
                if (httpsClient == null) {
                    synchronized (syncLock) {
                        if (httpsClient == null) {
                            httpsClient = YtoHttpClientBuilder.custom()
                                    .ssl()
                                    .timeout(CONNECT_TIMEOUT, SOCKET_TIMEOUT, CONNECT_REQUEST_TIMEOUT)
                                    .pool(MAX_TOTAL, MAX_PER_ROUTE)
                                    .retry(RETRY_TIMES, true).build();

                        }
                    }
                }
                httpRequestConfig.client(httpsClient);
            } else {
                if (httpClient == null) {
                    synchronized (syncLock) {
                        if (httpClient == null) {
                            httpClient = YtoHttpClientBuilder.custom()
                                    .timeout(CONNECT_TIMEOUT, SOCKET_TIMEOUT, CONNECT_REQUEST_TIMEOUT)
                                    .pool(MAX_TOTAL, MAX_PER_ROUTE)
                                    .retry(RETRY_TIMES, true).build();

                        }
                    }
                }
                httpRequestConfig.client(httpClient);
            }
        }
    }

    /**
     * GET 请求
     *
     * @param httpRequestConfig 请求配置类
     * @return 请求结果
     * @throws IOException
     */
    public static HttpRequestResult get(HttpRequestConfig httpRequestConfig) throws IOException {
        httpRequestConfig.addHeader("accept", "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9");
        httpRequestConfig.addHeader("Accept-Encoding", "gzip, deflate, br");
        httpRequestConfig.addHeader("Accept-Language", "zh-CN,zh;q=0.9,en;q=0.8,en-GB;q=0.7,en-US;q=0.6");
        httpRequestConfig.addHeader("User-Agent", "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/60.0.3112.90 Safari/537.36");
        return transformHttpRequestResult(execute(httpRequestConfig.requestMethod(HttpRequestMethod.GET)), httpRequestConfig);
    }

    /**
     * POST 请求
     *
     * @param httpRequestConfig
     * @return
     * @throws IOException
     */
    public static HttpRequestResult post(HttpRequestConfig httpRequestConfig) throws Exception {
        httpRequestConfig.addHeader("Accept", "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8");
        httpRequestConfig.addHeader("Accept-Encoding", "gzip, deflate");
        httpRequestConfig.addHeader("Accept-Language", "zh-CN,zh;q=0.8");
        httpRequestConfig.addHeader("User-Agent", "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/60.0.3112.90 Safari/537.36");
//        httpRequestConfig.addHeader("Content-Type", "text/html; charset=UTF-8");
        httpRequestConfig.addHeader(HTTP.CONN_DIRECTIVE, HTTP.CONN_CLOSE);

        return transformHttpRequestResult(execute(httpRequestConfig.requestMethod(HttpRequestMethod.POST)), httpRequestConfig);
    }

    /**
     * PUT 请求
     *
     * @param httpRequestConfig
     * @return
     * @throws IOException
     */
    public static HttpRequestResult put(HttpRequestConfig httpRequestConfig) throws IOException {
        return transformHttpRequestResult(execute(httpRequestConfig.requestMethod(HttpRequestMethod.PUT)), httpRequestConfig);
    }

    /**
     * HEADER 请求
     *
     * @param httpRequestConfig
     * @return
     * @throws IOException
     */
    public static HttpRequestResult header(HttpRequestConfig httpRequestConfig) throws IOException {
        return transformHttpRequestResult(execute(httpRequestConfig.requestMethod(HttpRequestMethod.HEAD)), httpRequestConfig);
    }

    /**
     * PATCH 请求
     *
     * @param httpRequestConfig
     * @return
     * @throws IOException
     */
    public static HttpRequestResult patch(HttpRequestConfig httpRequestConfig) throws IOException {
        return transformHttpRequestResult(execute(httpRequestConfig.requestMethod(HttpRequestMethod.PATCH)), httpRequestConfig);
    }

    /**
     * DELETE 请求
     *
     * @param httpRequestConfig
     * @return
     * @throws IOException
     */
    public static HttpRequestResult delete(HttpRequestConfig httpRequestConfig) throws IOException {
        return transformHttpRequestResult(execute(httpRequestConfig.requestMethod(HttpRequestMethod.DELETE)), httpRequestConfig);
    }


    /**
     * TRACE 请求
     *
     * @param httpRequestConfig
     * @return
     * @throws IOException
     */
    public static HttpRequestResult trace(HttpRequestConfig httpRequestConfig) throws IOException {
        return transformHttpRequestResult(execute(httpRequestConfig.requestMethod(HttpRequestMethod.TRACE)), httpRequestConfig);
    }

    /**
     * OPTIONS 请求
     *
     * @param httpRequestConfig
     * @return
     * @throws IOException
     */
    public static HttpRequestResult options(HttpRequestConfig httpRequestConfig) throws IOException {
        return transformHttpRequestResult(execute(httpRequestConfig.requestMethod(HttpRequestMethod.OPTIONS)), httpRequestConfig);
    }

    /**
     * 下载
     *
     * @param httpRequestConfig
     * @return
     * @throws IOException
     */
    public static OutputStream down(HttpRequestConfig httpRequestConfig) throws IOException {
        HttpResponse response = execute(httpRequestConfig.requestMethod(HttpRequestMethod.GET));
        response.getEntity().writeTo(HttpRequestConfig.getOut());
        close(response);
        return HttpRequestConfig.getOut();
    }


    /**
     * 请求资源或服务
     *
     * @param config 请求参数配置
     * @return 返回HttpResponse对象
     */
    private static HttpResponse execute(HttpRequestConfig config) throws IOException {

        if (config.getRequestMethod() == null) {
            throw new NullPointerException("request method is null!");
        }

        //设置httpClient
        setHttpClinet(config);

        //创建请求对象
        HttpRequestBase request = getRequest(HttpRequestConfig.getUrl(), config.getRequestMethod());

        //设置header信息
        request.setHeaders(config.getHeaders());

        request.setProtocolVersion(HttpVersion.HTTP_1_0);

        //判断是否支持设置entity(仅HttpPost、HttpPut、HttpPatch支持)
        if (HttpEntityEnclosingRequestBase.class.isAssignableFrom(request.getClass())) {
            //装填参数
            HttpEntity entity = config.getEntity();
            //设置参数到请求对象中
            ((HttpEntityEnclosingRequestBase) request).setEntity(entity);

            if (config.getParamMap().size() > 0) {
                log.debug("请求参数：{}", config.getParamMap());
            }

            if (StringUtils.isNotBlank(config.getBody())) {
                log.debug("请求参数：{}", config.getBody());
            }

        } else {
            int index = HttpRequestConfig.getUrl().indexOf("?");
            if (index > 0) {
                log.debug("请求参数：{}", HttpRequestConfig.getUrl().substring(index + 1));
            }
        }
        //执行请求操作，并拿到结果（同步阻塞）
        if (config.getContext() == null) {
            return config.getHttpClient().execute(request);
        } else {
            return config.getHttpClient().execute(request, config.getContext());
        }


    }

    private static HttpRequestResult transformHttpRequestResult(HttpResponse response, HttpRequestConfig config) throws IOException {
        //获取结果实体
        HttpRequestResult result = new HttpRequestResult(response.getStatusLine().getStatusCode(), response.getAllHeaders());
        if (response.getEntity() != null) {
            //content 编码
            String charset = (response.getEntity().getContentEncoding() == null) ? config.getResponseCharset() : response.getEntity().getContentEncoding().getValue();
            result.setContentEncoding(charset);
            //content 类型
            String contentType = (response.getEntity().getContentType() == null) ? "" : response.getEntity().getContentType().getValue();
            result.setContentType(contentType);
            result.setResponseText(EntityUtils.toString(response.getEntity(), config.getResponseCharset()));
        } else {//有可能是head请求
            result.setResponseText(response.getStatusLine().toString());
        }
        close(response);
        return result;
    }

    /**
     * 尝试关闭
     *
     * @param response
     */
    public static void close(HttpResponse response) {
        try {
            EntityUtils.consume(response.getEntity());
            if (response != null) {
                //如果CloseableHttpResponse 是resp的父类，则支持关闭
                if (CloseableHttpResponse.class.isAssignableFrom(response.getClass())) {
                    ((CloseableHttpResponse) response).close();
                }
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * 根据请求方法名，获取request对象
     *
     * @param url    资源地址
     * @param method 请求方式
     * @return 返回Http处理request基类
     */
    private static HttpRequestBase getRequest(String url, HttpRequestMethod method) {
        HttpRequestBase request = null;
        switch (method.getCode()) {
            case 0:
                request = new HttpGet(url);
                break;
            case 1:
                request = new HttpPost(url);
                break;
            case 2:
                request = new HttpHead(url);
                break;
            case 3:
                request = new HttpPut(url);
                break;
            case 4:
                request = new HttpDelete(url);
                break;
            case 5:
                request = new HttpTrace(url);
                break;
            case 6:
                request = new HttpPatch(url);
                break;
            case 7:
                request = new HttpOptions(url);
                break;
            default:
                request = new HttpPost(url);
                break;
        }
        return request;
    }
}